/************************************************************************
 *   IRC - Internet Relay Chat, m_services.c
 *   Copyright (C) 2005 PleXusIRCd Development Team
 *   Copyright (C) 2005 IRCD-Hybrid Team
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   $Id: m_services.c 88 2005-11-19 23:43:15Z jon $
 */

/*
 *
 * With code from 'bane' the URL following is valid as of this date.
 * http://hybserv2.net/file/trunk/contrib/hybrid_services.c
 *
 *
 *   Copyright (C) 2002, 2003, 2004, 2005 by Dragan 'bane' Dosen and the
 *   Hybrid Development Team.
 *
 *   Based on m_services.c, originally from bahamut ircd.
 *
 *   Contact info:
 *
 *     E-mail : bane@idolnet.org
 *     IRC    : (*) bane, idolNET, irc.idolnet.org, #twilight_zone
 */

#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "channel.h"
#include "channel_mode.h"
#include "common.h"
#include "ircd.h"
#include "irc_string.h"
#include "numeric.h"
#include "fdlist.h"
#include "s_bsd.h"
#include "s_conf.h"
#include "s_log.h"
#include "s_serv.h"
#include "s_user.h"
#include "send.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"
#include "hash.h"
#include "whowas.h"

static void m_ns(struct Client *, struct Client *, int, char *[]);
static void m_cs(struct Client *, struct Client *, int, char *[]);
static void m_os(struct Client *, struct Client *, int, char *[]);
static void m_ms(struct Client *, struct Client *, int, char *[]);
static void m_bs(struct Client *, struct Client *, int, char *[]);
static void m_hs(struct Client *, struct Client *, int, char *[]);
static void m_identify(struct Client *, struct Client *, int, char *[]);
static void ms_svshost(struct Client *, struct Client *, int, char *[]);
static void ms_svsid(struct Client *, struct Client *, int, char *[]);
static void ms_svsjoin(struct Client *, struct Client *, int, char *[]);
static void ms_svsmode(struct Client *, struct Client *, int, char *[]);
static void ms_svsnick(struct Client *, struct Client *, int, char *[]);
static void ms_svspart(struct Client *, struct Client *, int, char *[]);
static void ms_svstopic(struct Client *, struct Client *, int, char *[]);
static void get_string(int, char *[], char *);
static void deliver_services_msg(const char *, const char *, struct Client *,
				 struct Client *, int, char *[]);
static int clean_nick_name(char *);

/* Custom Macros */
#define services_function(a,b,c) static void a(struct Client *client_p,\
        struct Client *source_p, int parc,\
        char *parv[]) \
{ deliver_services_msg(b, c, client_p, source_p, parc, parv); }

struct Message ns_msgtab = {
	"NS", 0, 0, 1, 0, MFLG_SLOW, 0,
	{m_ignore, m_ns, m_ignore, m_ignore, m_ns}
};
struct Message nickserv_msgtab = {
	"NICKSERV", 0, 0, 1, 0, MFLG_SLOW, 0,
	{m_ignore, m_ns, m_ignore, m_ignore, m_ns}
};
struct Message cs_msgtab = {
	"CS", 0, 0, 1, 0, MFLG_SLOW, 0,
	{m_ignore, m_cs, m_ignore, m_ignore, m_cs}
};
struct Message chanserv_msgtab = {
	"CHANSERV", 0, 0, 1, 0, MFLG_SLOW, 0,
	{m_ignore, m_cs, m_ignore, m_ignore, m_cs}
};
struct Message os_msgtab = {
	"OS", 0, 0, 1, 0, MFLG_SLOW, 0,
	{m_ignore, m_os, m_ignore, m_ignore, m_os}
};
struct Message operserv_msgtab = {
	"OPERSERV", 0, 0, 1, 0, MFLG_SLOW, 0,
	{m_ignore, m_os, m_ignore, m_ignore, m_os}
};
struct Message ms_msgtab = {
	"MS", 0, 0, 1, 0, MFLG_SLOW, 0,
	{m_ignore, m_ms, m_ignore, m_ignore, m_ms}
};
struct Message memoserv_msgtab = {
	"MEMOSERV", 0, 0, 1, 0, MFLG_SLOW, 0,
	{m_ignore, m_ms, m_ignore, m_ignore, m_ms}
};
struct Message bs_msgtab = {
	"BS", 0, 0, 1, 0, MFLG_SLOW, 0,
	{m_ignore, m_bs, m_ignore, m_ignore, m_bs}
};
struct Message botserv_msgtab = {
	"BOTSERV", 0, 0, 1, 0, MFLG_SLOW, 0,
	{m_ignore, m_bs, m_ignore, m_ignore, m_bs}
};
struct Message hs_msgtab = {
	"HS", 0, 0, 1, 0, MFLG_SLOW, 0,
	{m_ignore, m_hs, m_ignore, m_ignore, m_hs}
};
struct Message hostserv_msgtab = {
	"HOSTSERV", 0, 0, 1, 0, MFLG_SLOW, 0,
	{m_ignore, m_hs, m_ignore, m_ignore, m_hs}
};
struct Message identify_msgtab = {
	"IDENTIFY", 0, 0, 2, 0, MFLG_SLOW, 0,
	{m_ignore, m_identify, m_ignore, m_ignore, m_identify}
};
struct Message svshost_msgtab = {
	"SVSHOST", 0, 0, 3, 0, MFLG_SLOW, 0,
	{m_ignore, m_ignore, ms_svshost, m_ignore, m_ignore}
};
struct Message svsid_msgtab = {
	"SVSID", 0, 0, 3, 0, MFLG_SLOW, 0,
	{m_ignore, m_ignore, ms_svsid, m_ignore, m_ignore}
};
struct Message svsjoin_msgtab = {
	"SVSJOIN", 0, 0, 3, 0, MFLG_SLOW, 0,
	{m_ignore, m_ignore, ms_svsjoin, m_ignore, m_ignore}
};
struct Message svsmode_msgtab = {
	"SVSMODE", 0, 0, 3, 0, MFLG_SLOW, 0,
	{m_ignore, m_ignore, ms_svsmode, m_ignore, m_ignore}
};
struct Message svsnick_msgtab = {
	"SVSNICK", 0, 0, 3, 0, MFLG_SLOW, 0,
	{m_unregistered, m_ignore, ms_svsnick, m_ignore, m_ignore}
};
struct Message svspart_msgtab = {
	"SVSPART", 0, 0, 3, 0, MFLG_SLOW, 0,
	{m_ignore, m_ignore, ms_svspart, m_ignore, m_ignore}
};
struct Message svstopic_msgtab = {
	"SVSTOPIC", 0, 0, 2, 0, MFLG_SLOW, 0,
	{m_unregistered, m_ignore, ms_svstopic, m_ignore, m_ignore}
};

#ifndef STATIC_MODULES
void
_modinit(void)
{
	mod_add_cmd(&ns_msgtab);
	mod_add_cmd(&nickserv_msgtab);
	mod_add_cmd(&cs_msgtab);
	mod_add_cmd(&chanserv_msgtab);
	mod_add_cmd(&os_msgtab);
	mod_add_cmd(&operserv_msgtab);
	mod_add_cmd(&ms_msgtab);
	mod_add_cmd(&memoserv_msgtab);
	mod_add_cmd(&bs_msgtab);
	mod_add_cmd(&botserv_msgtab);
	mod_add_cmd(&hs_msgtab);
	mod_add_cmd(&hostserv_msgtab);
	mod_add_cmd(&identify_msgtab);
	mod_add_cmd(&svshost_msgtab);
	mod_add_cmd(&svsid_msgtab);
	mod_add_cmd(&svsjoin_msgtab);
	mod_add_cmd(&svsmode_msgtab);
	mod_add_cmd(&svsnick_msgtab);
	mod_add_cmd(&svspart_msgtab);
	mod_add_cmd(&svstopic_msgtab);
}

void
_moddeinit(void)
{
	mod_del_cmd(&ns_msgtab);
	mod_del_cmd(&nickserv_msgtab);
	mod_del_cmd(&cs_msgtab);
	mod_del_cmd(&chanserv_msgtab);
	mod_del_cmd(&os_msgtab);
	mod_del_cmd(&operserv_msgtab);
	mod_del_cmd(&ms_msgtab);
	mod_del_cmd(&memoserv_msgtab);
	mod_del_cmd(&bs_msgtab);
	mod_del_cmd(&botserv_msgtab);
	mod_del_cmd(&hs_msgtab);
	mod_del_cmd(&hostserv_msgtab);
	mod_del_cmd(&identify_msgtab);
	mod_del_cmd(&svshost_msgtab);
	mod_del_cmd(&svsid_msgtab);
	mod_del_cmd(&svsjoin_msgtab);
	mod_del_cmd(&svsmode_msgtab);
	mod_del_cmd(&svsnick_msgtab);
	mod_del_cmd(&svspart_msgtab);
	mod_del_cmd(&svstopic_msgtab);
}

const char *_version = "$Revision: 88 $";
#endif

/*
 * These generate the services functions through
 * a macro.
 */
services_function(m_bs, "BotServ", "BOTSERV")
services_function(m_cs, "ChanServ", "CHANSERV")
services_function(m_hs, "HostServ", "HOSTSERV")
services_function(m_ms, "MemoServ", "MEMOSERV")
services_function(m_ns, "NickServ", "NICKSERV")
services_function(m_os, "OperServ", "OPERSERV")

/*
 * get_string() 
 *
 * Reverse the array parv back into a normal string assuming
 * there are "parc" indicies in the array.
 *
 * Originally GetString() written by sidewndr.
 * Modified by Michael for use with hybrid-7.
 */
static void
get_string(int parc, char *parv[], char *buf)
{
	int ii = 0;
	int bw = 0;

	for (; ii < parc; ++ii)
		bw += ircsprintf(buf + bw, "%s ", parv[ii]);
	buf[bw - 1] = '\0';
}

/*
 * deliver_services_msg()
 *
 * parv[0] = sender prefix
 * servmsg = message for services (utilising GetString())
 *
 * Borrowed from HybServ -- knight-
 */
static void
deliver_services_msg(const char *service, const char *command,
		     struct Client *client_p, struct Client *source_p, int parc, char *parv[])
{
	struct Client *target_p = NULL;
	char buf[IRCD_BUFSIZE] = { '\0' };

	if(parc < 2 || *parv[1] == '\0')
	{
		sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS),
			   me.name, source_p->name, command);
		return;
	}

	if(!(target_p = find_server(SERVICES)))
		sendto_one(source_p, form_str(ERR_SERVICESDOWN), me.name, source_p->name, service);
	else
	{
		get_string(parc - 1, parv + 1, buf);
		sendto_one(target_p, ":%s PRIVMSG %s@%s :%s",
			   source_p->name, service, SERVICES, buf);
	}
}

/*
 * m_identify()
 *
 * parv[0] = sender prefix
 * parv[1] = NickServ Password or Channel
 * parv[2] = ChanServ Password
 */
static void
m_identify(struct Client *client_p, struct Client *source_p, int parc, char *parv[])
{
	struct Client *target_p = NULL;

	switch (parc)
	{
	case 2:
		if(!(target_p = find_server(SERVICES)))
			sendto_one(source_p, form_str(ERR_SERVICESDOWN),
				   me.name, source_p->name, "NickServ");
		else
			sendto_one(target_p, ":%s PRIVMSG NickServ@%s :IDENTIFY %s",
				   source_p->name, SERVICES, parv[1]);
		break;

	case 3:
		if(!(target_p = find_server(SERVICES)))
			sendto_one(source_p, form_str(ERR_SERVICESDOWN),
				   me.name, source_p->name, "ChanServ");
		else
			sendto_one(target_p, ":%s PRIVMSG ChanServ@%s :IDENTIFY %s %s",
				   source_p->name, SERVICES, parv[1], parv[2]);
		break;

	default:
		sendto_one(source_p, ":%s NOTICE %s :Syntax: IDENTIFY <password> "
			   "- for nickname", me.name, source_p->name);
		sendto_one(source_p, ":%s NOTICE %s :Syntax: IDENTIFY <channel> "
			   "<password> - for channel", me.name, source_p->name);
		break;
	}
}

/*
 * ms_svshost:
 *      Changes the Targets Virtual Hostname, and also sets +h if its not set already on the target
 *      parv[0] = sender prefix
 *      parv[1] = target
 *      parv[2] = new hostname
 */
static void
ms_svshost(struct Client *client_p, struct Client *source_p, int parc, char *parv[])
{
	struct Client *target_p;
	int setflags;

	if(parc < 3)
		return;

	if(!IsServer(source_p) && !strcmp(SERVICES, source_p->name))
		return;

	if(!valid_hostname(parv[2]))
		return;

	sendto_server(client_p, source_p, NULL, NOCAPS, NOCAPS, LL_ICLIENT,
		      ":%s SVSHOST %s :%s", parv[0], parv[1], parv[2]);

	target_p = find_person(parv[1]);

	if(target_p == NULL)
		return;

	strlcpy(target_p->vhost, parv[2], sizeof(target_p->vhost));

	if(MyClient(target_p))
		setflags = target_p->umodes;

	if(!IsHidden(target_p))
		SetHidden(target_p);

	if(MyClient(target_p) && (setflags != target_p->umodes))
	{
		char buf[BUFSIZE];
		send_umode(target_p, target_p, setflags, ALL_UMODES, buf);
	}
}

/*
 * ms_svsid:
 *      Sets/Changes the services ID for a particular nickname
 *      parv[0] = sender prefix
 *      parv[1] = target 
 *      parv[2] = newsvsid
 */
static void
ms_svsid(struct Client *client_p, struct Client *source_p, int parc, char *parv[])
{
	struct Client *target_p;

	if(parc < 3)
		return;

	if(!IsServer(source_p) && !strcmp(SERVICES, source_p->name))
		return;

	sendto_server(client_p, source_p, NULL, NOCAPS, NOCAPS, LL_ICLIENT,
		      ":%s SVSID %s %s", parv[0], parv[1], parv[2]);

	target_p = find_person(parv[1]);

	if(target_p == NULL)
		return;

	target_p->svsid = atol(parv[2]);
}

/*
 * ms_svsjoin:
 * 	Causes the target to join channel with optional status
 *      parv[0] = sender prefix
 *      parv[1] = nickname to join 
 *      parv[2] = channel to join them into
 */
static void
ms_svsjoin(struct Client *client_p, struct Client *source_p, int parc, char *parv[])
{
	struct Client *target_p;
	struct Channel *chptr;
	int type;
	char mode;
	char sjmode;
	char *newch;

	if(!IsServer(source_p) && !strcmp(SERVICES, source_p->name))
		return;

	if((hunt_server(client_p, source_p, ":%s SVSJOIN %s %s", 1, parc, parv)) != HUNTED_ISME)
		return;

	if((target_p = find_client(parv[1])) == NULL)
		return;

	if(!IsPerson(target_p))
		return;

	if(*parv[2] == '@')
	{
		type = CHFL_CHANOP;
		mode = 'o';
		sjmode = '@';
	}
#ifdef HALFOPS
	else if(*parv[2] == '%')
	{
		type = CHFL_HALFOP;
		mode = 'h';
		sjmode = '%';
	}
#endif
	else if(*parv[2] == '+')
	{
		type = CHFL_VOICE;
		mode = 'v';
		sjmode = '+';
	}
	else
	{
		type = CHFL_PEON;
		mode = sjmode = '\0';
	}

	if(mode != '\0')
		parv[2]++;

	if((chptr = hash_find_channel(parv[2])) != NULL)
	{
		if(IsMember(target_p, chptr))
			return;

		add_user_to_channel(chptr, target_p, type);

		if(chptr->chname[0] != '&')
			sendto_server(target_p, target_p, chptr, NOCAPS, NOCAPS, LL_ICLIENT,
				      ":%s SJOIN %lu %s + :%c%s",
				      me.name, (unsigned long) chptr->channelts,
				      chptr->chname, type ? sjmode : ' ', target_p->name);

		sendto_channel_local(ALL_MEMBERS, chptr, ":%s!%s@%s JOIN :%s",
				     target_p->name, target_p->username,
				     target_p->vhost, chptr->chname);

		if(type)
			sendto_channel_local(ALL_MEMBERS, chptr, ":%s MODE %s +%c %s",
					     me.name, chptr->chname, mode, target_p->name);

		if(chptr->topic != NULL)
		{
			sendto_one(target_p, form_str(RPL_TOPIC), me.name,
				   target_p->name, chptr->chname, chptr->topic);
			sendto_one(target_p, form_str(RPL_TOPICWHOTIME),
				   me.name, source_p->name, chptr->chname,
				   chptr->topic_info, chptr->topic_time);
		}

		channel_member_names(target_p, chptr, chptr->chname, 1);
	}
	else
	{
		newch = parv[2];
		if(!check_channel_name(newch))
			return;

		if(!IsChannelName(newch))
			return;

		if(ConfigServerHide.disable_local_channels && (*newch == '&'))
			return;

		if(strlen(newch) > CHANNELLEN)
			return;

		chptr = get_or_create_channel(target_p, newch, NULL);
		add_user_to_channel(chptr, target_p, CHFL_CHANOP);

		if(chptr->chname[0] != '&')
			sendto_server(target_p, target_p, chptr, NOCAPS, NOCAPS, LL_ICLIENT,
				      ":%s SJOIN %lu %s +ntZ :@%s", me.name,
				      (unsigned long) chptr->channelts, chptr->chname,
				      target_p->name);

		sendto_channel_local(ALL_MEMBERS, chptr, ":%s!%s@%s JOIN :%s",
				     target_p->name, target_p->username,
				     target_p->vhost, chptr->chname);

		chptr->mode.mode |= MODE_TOPICLIMIT;
		chptr->mode.mode |= MODE_NOPRIVMSGS;
		chptr->mode.mode |= MODE_BW;

		sendto_channel_local(ALL_MEMBERS, chptr, ":%s MODE %s +ntZ", me.name,
				     chptr->chname);

		target_p->localClient->last_join_time = CurrentTime;
		channel_member_names(target_p, chptr, chptr->chname, 1);
	}
}

/*
 * ms_svsmode:
 *      Sets/Changes modes for a particular nickname
 *      parv[0] = sender prefix
 *      parv[1] = target 
 *      parv[2] = modes
 */
static void
ms_svsmode(struct Client *client_p, struct Client *source_p, int parc, char *parv[])
{
	struct Client *target_p;

	if(parc < 3)
		return;

	target_p = find_person(parv[1]);

	if(target_p == NULL)
		return;

	user_mode(client_p, source_p, parc, parv);
}

/*
 * ms_svsnick:
 *	Changes target's nickname
 *	parv[0] = sender prefix
 *	parv[1] = oldnick
 *	parv[2] = newnick
 */
static void
ms_svsnick(struct Client *client_p, struct Client *source_p, int parc, char *parv[])
{
	char oldnick[NICKLEN];
	char newnick[NICKLEN];
	struct Client *oldnickname;
	struct Client *newnickname;

	if(parc < 3 || BadPtr(parv[1]) || BadPtr(parv[2]))
		return;

	if(!IsServer(source_p))
		return;

	if((hunt_server(client_p, source_p, ":%s SVSNICK %s %s", 1, parc, parv)) != HUNTED_ISME)
		return;

	strlcpy(oldnick, parv[1], NICKLEN);
	strlcpy(newnick, parv[2], NICKLEN);

	if(!clean_nick_name(oldnick) || !clean_nick_name(newnick))
		return;

	if(!(oldnickname = find_client(oldnick)))
		return;

	if(newnickname = find_client(newnick))
		return;

	if(!IsFloodDone(oldnickname))
		flood_endgrace(oldnickname);

	if(newnickname = find_client(oldnick))
	{
		if(newnickname == oldnickname)
		{
			if(strcmp(oldnickname->name, newnick))
			{
				change_local_nick(oldnickname->servptr, oldnickname, newnick, 1);
				return;
			}
			else
			{
				return;
			}
		}
		if(IsUnknown(newnickname))
		{
			exit_client(NULL, newnickname, &me, "SVSNICK Override");
			change_local_nick(oldnickname->servptr, oldnickname, newnick, 1);
			return;
		}
		else
		{
			return;
		}
	}
	else
	{
		if(!ServerInfo.hub && uplink && IsCapable(uplink, CAP_LL))
		{
			sendto_one(uplink, ":%s NBURST %s %s %s", me.name, newnick,
				   newnick, oldnickname->name);
			return;
		}
		else
		{
			change_local_nick(oldnickname->servptr, oldnickname, newnick, 1);
			return;
		}
	}
}

/*
 * ms_svspart:
 *      Causes the target to part channel
 *      parv[0] = sender prefix
 *      parv[1] = target nickname
 *      parv[2] = channel to part
 */
static void
ms_svspart(struct Client *client_p, struct Client *source_p, int parc, char *parv[])
{
	struct Client *target_p;
	struct Channel *chptr;

	if(!IsServer(source_p) && !strcmp(SERVICES, source_p->name))
		return;

	if((hunt_server(client_p, source_p, ":%s SVSPART %s %s", 1, parc, parv)) != HUNTED_ISME)
		return;

	if((target_p = find_client(parv[1])) == NULL)
		return;

	if(!IsClient(target_p))
		return;

	if((chptr = hash_find_channel(parv[2])) == NULL)
		return;

	if(!IsMember(target_p, chptr))
		return;

	if(chptr->chname[0] != '&')
		sendto_server(target_p, target_p, chptr, NOCAPS, NOCAPS, LL_ICLIENT,
			      ":%s PART %s :%s", target_p->name, chptr->chname, target_p->name);

	sendto_channel_local(ALL_MEMBERS, chptr, ":%s!%s@%s PART %s :%s",
			     target_p->name, target_p->username,
			     target_p->vhost, chptr->chname, target_p->name);
	remove_user_from_channel(chptr, target_p);
}

/*
 * ms_svstopic:
 * 	Let services always set a topic
 *      parv[0] = sender prefix
 *      parv[1] = channel name
 *      parv[2] = topic_info
 *      parv[3] = topic_info time
 *      parv[4] = new channel topic
 */
static void
ms_svstopic(struct Client *client_p, struct Client *source_p, int parc, char *parv[])
{
	struct Channel *chptr = NULL;

	if(parc < 5)
		return;

	if(!IsServices(source_p))
		return;

	if(parv[1] && !IsChannelName(parv[1]))
		return;

	if((chptr = hash_find_channel(parv[1])) == NULL)
		return;

	if((chptr->topic != NULL) && !strcmp(chptr->topic, parv[4]))
		return;

	sendto_server(client_p, source_p, chptr, NOCAPS, NOCAPS, LL_ICLIENT,
		      ":%s SVSTOPIC %s %s %s :%s", parv[0], parv[1], parv[2], parv[3], parv[4]);

	set_channel_topic(chptr, parv[4], parv[2], atoi(parv[3]));

	sendto_channel_local(ALL_MEMBERS, chptr, ":%s TOPIC %s :%s",
			     source_p->name, chptr->chname,
			     chptr->topic == NULL ? "" : chptr->topic);
}

/*
 * clean_nick_name()                 
 *
 * input        - nickname
 * output       - none
 * side effects - walks through the nickname, returning 0 if erroneous
 */
static int
clean_nick_name(char *nick)
{
	assert(nick);
	if(nick == NULL)
		return (0);

	if(*nick == '-' || IsDigit(*nick) || *nick == '\0')
		return (0);

	for (; *nick; nick++)
	{
		if(!IsNickChar(*nick))
			return (0);
	}

	return (1);
}
