/************************************************************************
 *   IRC - Internet Relay Chat, m_sethost.c
 *   Copyright (C) 2003 PleXusIRCd Development Team
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   $Id: m_sethost.c 76 2005-10-04 03:47:51Z jon $
 */

#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "common.h"		/* FALSE bleah */
#include "ircd.h"
#include "irc_string.h"
#include "numeric.h"
#include "fdlist.h"
#include "s_bsd.h"
#include "s_conf.h"
#include "s_log.h"
#include "s_serv.h"
#include "send.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"

static void ms_sethost(struct Client *client_p, struct Client *source_p, int parc, char *parv[]);

struct Message sethost_msgtab = {
	"SETHOST", 0, 0, 3, 0, MFLG_SLOW, 0,
	{m_ignore, m_ignore, ms_sethost, m_ignore, m_ignore}
};

#ifndef STATIC_MODULES
void
_modinit(void)
{
	mod_add_cmd(&sethost_msgtab);
}

void
_moddeinit(void)
{
	mod_del_cmd(&sethost_msgtab);
}

const char *_version = "$Revision: 76 $";
#endif

/*
 * ms_sethost: Changes the targets virtual hostname
 *
 * parv[0] = sender prefix
 * parv[1] = target
 * parv[2] = new hostname
 *
 */
static void
ms_sethost(struct Client *client_p, struct Client *source_p, int parc, char *parv[])
{
	struct Client *target_p;

	if(parc < 3)
		return;

	if(!IsServer(source_p))
		return;

	sendto_server(client_p, source_p, NULL, NOCAPS, NOCAPS, LL_ICLIENT,
		      ":%s SETHOST %s :%s", parv[0], parv[1], parv[2]);

	target_p = find_person(parv[1]);

	if(target_p == NULL)
		return;

	strlcpy(target_p->vhost, parv[2], sizeof(target_p->vhost));
}
